package org.prlab.idic.privacypolicy.storage;

import android.content.ContentResolver;
import android.content.ContentValues;
import android.database.Cursor;
import android.net.Uri;
import android.util.Log;

import org.prlab.idic.privacypolicy.tool.BaseUtils;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

/**
 * Created by IDIC on 2017/2/15.
 */
public class PrivacyPolicyStorageImpl implements StorageInterface<Integer, String> {

    private static final String TAG = PrivacyPolicyStorageImpl.class.getSimpleName();

    private ContentResolver mContentResolver;
    private String mTargetDeviceMAC;

    private static final String[] PRIVACY_POLICY_PROJECTION = {
            PrivacyPolicyContract.PolicyEntry.COLUMN_MAC,
            PrivacyPolicyContract.PolicyEntry.COLUMN_UUIDs,
            PrivacyPolicyContract.PolicyEntry.COLUMN_PREFERENCE
    };

    public static final int INDEX_POLICY_MAC = 0;
    public static final int INDEX_POLICY_UUIDs = 1;
    public static final int INDEX_POLICY_PREFERENCE = 2;

    public PrivacyPolicyStorageImpl(ContentResolver contentResolver, String targetDeviceMAC) {
        this.mContentResolver = contentResolver;
        this.mTargetDeviceMAC = BaseUtils.shortMAC(targetDeviceMAC);
    }

    @Override
    public StorageInterface remove(List<String> layerUUIDs) {
        remove(BaseUtils.stringList2ArrayString(layerUUIDs));
        return this;
    }

    private void remove(String layerUUIDs) {
        int impactRow = mContentResolver.delete(
                PrivacyPolicyContract.PolicyEntry.CONTENT_URI,
                PrivacyPolicyContract.getSqlSelectForMACAndUUIDsOnwards(),
                new String[]{mTargetDeviceMAC, layerUUIDs}
        );
        removeChildLayerPolicies(layerUUIDs);
        Log.i(TAG, "Remove [" + layerUUIDs + "] " + (impactRow == 1 ? "success" : "failure") + ".");
    }

    private void removeChildLayerPolicies(String layerUUIDs) {
        List<String> subUUIDs = getRelatedUUIDs(true, layerUUIDs);
        for (String subUUID : subUUIDs) {
            remove(subUUID);
        }
    }

    @Override
    public StorageInterface put(List<Integer> selection, List<String> layerUUIDs) {
        put(selection, BaseUtils.stringList2ArrayString(layerUUIDs));
        return this;
    }

    private void put(List<Integer> selection, String layerUUIDs) {
        for (int s : selection) {
            ContentValues updateContentValues = new ContentValues();
            updateContentValues.put(PrivacyPolicyContract.PolicyEntry.COLUMN_PREFERENCE, s);

            ContentValues insertContentValues = new ContentValues(updateContentValues);
            insertContentValues.put(PrivacyPolicyContract.PolicyEntry.COLUMN_MAC, mTargetDeviceMAC);
            insertContentValues.put(PrivacyPolicyContract.PolicyEntry.COLUMN_UUIDs, layerUUIDs);
            Uri mNewUri = mContentResolver.insert(
                    PrivacyPolicyContract.PolicyEntry.CONTENT_URI,
                    insertContentValues
            );
            if (mNewUri == null) {
                int count = mContentResolver.update(
                        PrivacyPolicyContract.PolicyEntry.CONTENT_URI,
                        updateContentValues,
                        PrivacyPolicyContract.getSqlSelectForMACAndUUIDsOnwards(),
                        new String[]{mTargetDeviceMAC, layerUUIDs}
                );
                Log.i(TAG, "Update " + String.valueOf(s) + " to [" + layerUUIDs + "] " + (count == 1 ? "success" : "failure") + ".");
            } else {
                Log.i(TAG, "Put " + String.valueOf(s) + " to [" + layerUUIDs + "] success.");
            }

            removeChildLayerPolicies(layerUUIDs);
        }
    }

    @Override
    public List<Integer> get(List<String> layerUUIDs) {
        return get(BaseUtils.stringList2ArrayString(layerUUIDs));
    }

    private List<Integer> get(String UUIDs) {
        try (Cursor cursor = mContentResolver.query(
                PrivacyPolicyContract.PolicyEntry.CONTENT_URI,
                PRIVACY_POLICY_PROJECTION,
                PrivacyPolicyContract.getSqlSelectForMACAndUUIDsOnwards(),
                new String[]{mTargetDeviceMAC, UUIDs},
                PrivacyPolicyContract.PolicyEntry.COLUMN_UUIDs + " ASC")) {

            List<Integer> targets = new LinkedList<>();
            Set<Integer> targetSet = new HashSet<>();
            if (cursor != null && cursor.getCount() > 0) {
                cursor.moveToFirst();
                do {
                    String target = cursor.getString(INDEX_POLICY_PREFERENCE);
                    Log.i(TAG, "Get [" + UUIDs + "] -> [" + String.valueOf(target) + "]");
                    if (target == null) return null;
                    targetSet.add(Integer.valueOf(target));
                } while (cursor.moveToNext());
            }
            targetSet.addAll(getChildLayerPolicies(UUIDs));
            targets.addAll(targetSet);
            Log.i(TAG, "Get [" + UUIDs + "] -> null");
            return targets;
        }
    }

    private List<Integer> getChildLayerPolicies(String layerUUIDs) {
        List<Integer> selections = new LinkedList<>();
        List<Integer> selectionSet = new LinkedList<>();
        List<String> subUUIDs = getRelatedUUIDs(true, layerUUIDs);
        for (String subUUID : subUUIDs) {
            List<Integer> subSelections = get(subUUID);
            if (subSelections != null)
                selectionSet.addAll(subSelections);
        }
        selections.addAll(selectionSet);
        return selections;
    }

    private List<String> getRelatedUUIDs(boolean isLower, String UUIDs) {
        List<String> UUIDList = new LinkedList<>();
        if (isLower) {
            try (Cursor cursor = mContentResolver.query(
                    PrivacyPolicyContract.PolicyEntry.CONTENT_URI,
                    PRIVACY_POLICY_PROJECTION,
                    PrivacyPolicyContract.getSqlSelectForMACAndRelatedUUIDsOnwards(),
                    new String[]{mTargetDeviceMAC, UUIDs + "_%"},
                    PrivacyPolicyContract.PolicyEntry.COLUMN_UUIDs + " ASC")) {
                if (cursor != null)
                    while (cursor.moveToNext()) {
                        String targetUUIDs = cursor.getString(INDEX_POLICY_UUIDs);
                        UUIDList.add(targetUUIDs);
                    }
            }
        } else {
            if (UUIDs.contains(", ")) {
                UUIDs = UUIDs.substring(0, UUIDs.lastIndexOf(", "));
                try (Cursor cursor = mContentResolver.query(
                        PrivacyPolicyContract.PolicyEntry.CONTENT_URI,
                        PRIVACY_POLICY_PROJECTION,
                        PrivacyPolicyContract.getSqlSelectForMACAndRelatedUUIDsOnwards(),
                        new String[]{mTargetDeviceMAC, "%" + UUIDs},
                        PrivacyPolicyContract.PolicyEntry.COLUMN_UUIDs + " ASC")) {
                    if (cursor != null)
                        while (cursor.moveToNext()) {
                            String targetUUIDs = cursor.getString(INDEX_POLICY_UUIDs);
                            UUIDList.add(targetUUIDs);
                        }
                }
            }
        }
        return UUIDList;
    }

}
